@@ -39,10 +39,10 @@ class ActivatorTest:
 
         # Then
         activity = Activity.query.filter(Activity.changed_data['id'].astext.cast(Integer) == offer.id).one()
-        assert offer.activityUuid != activity.uuid
-        assert activity.verb == 'insert'
+        assert offer.activityUuid == activity.uuid
         assert activity.oldDatum == None
         assert activity.transaction == None
+        assert activity.verb == 'insert'
         assert offer_dict.items() <= activity.patch.items()
         assert offer_dict.items() <= activity.datum.items()
         assert activity.datum['id'] == humanize(offer.id)
@@ -62,8 +62,8 @@ class ActivatorTest:
 
         # Then
         activity = Activity.query.filter(Activity.changed_data['id'].astext.cast(Integer) == offer.id).one()
-        assert activity.verb == 'insert'
         assert activity.transaction.actor.id == user.id
+        assert activity.verb == 'insert'
 
     @with_delete
     def test_modify_offer_saves_an_update_activity(self, app):
@@ -84,12 +84,13 @@ class ActivatorTest:
             (Activity.data['id'].astext.cast(Integer) == offer.id)
         ).one()
         assert activity.verb == 'update'
+        assert activity.uuid == offer.activityUuid
         assert {**offer_dict, **modify_dict}.items() <= activity.datum.items()
         assert modify_dict.items() == activity.patch.items()
         assert offer_dict.items() <= activity.oldDatum.items()
 
     @with_delete
-    def test_create_activity_offer_saves_an_insert_activity(self, app):
+    def test_create_activity_on_not_existing_offer_saves_an_insert_activity(self, app):
         # Given
         offer_uuid = uuid4()
         patch = { 'name': 'bar', 'type': 'foo' }
@@ -104,6 +105,7 @@ class ActivatorTest:
         # Then
         activity = Activity.query.filter_by(uuid=offer_uuid).one()
         offer = Offer.query.filter_by(activityUuid=offer_uuid).one()
+        assert activity.uuid == offer.activityUuid
         assert activity.verb == 'insert'
         assert patch.items() <= activity.datum.items()
         assert patch.items() <= activity.patch.items()
@@ -111,7 +113,7 @@ class ActivatorTest:
         assert activity.patch['id'] == humanize(offer.id)
 
     @with_delete
-    def test_create_two_activities_offer_saves_an_insert_and_an_update_activity(self, app):
+    def test_create_activities_on_existing_offer_saves_none_activities_and_an_update_one(self, app):
         # Given
         offer_uuid = uuid4()
         first_patch = { 'name': 'bar', 'type': 'foo' }
@@ -124,15 +126,33 @@ class ActivatorTest:
                                    patch=second_patch,
                                    tableName='offer',
                                    uuid=offer_uuid)
+        third_patch = { 'type': 'fee' }
+        third_activity = Activity(dateCreated=datetime.utcnow(),
+                                  patch=third_patch,
+                                  tableName='offer',
+                                  uuid=offer_uuid)
 
         # When
-        ApiHandler.activate(first_activity, second_activity)
+        ApiHandler.activate(first_activity, second_activity, third_activity)
 
         # Then
         activities = Activity.query.filter_by(uuid=offer_uuid).all()
         offer = Offer.query.filter_by(activityUuid=offer_uuid).one()
-        assert len(activities) == 2
+        assert len(activities) == 4
+        assert activities[0].uuid == offer.activityUuid
+        assert activities[0].verb == 'insert'
+        assert activities[1].uuid == offer.activityUuid
+        assert activities[1].verb == None
+        assert activities[1].patch.items() == second_patch.items()
+        assert activities[2].uuid == offer.activityUuid
+        assert activities[2].verb == None
+        assert activities[2].patch.items() == third_patch.items()
+        assert activities[3].uuid == offer.activityUuid
+        assert activities[3].verb == 'update'
+        merged_patch = { 'name': 'bor', 'type': 'fee' }
+        assert activities[3].patch.items() == merged_patch.items()
         assert offer.name == 'bor'
+        assert offer.type == 'fee'
 
 
     @with_delete
@@ -157,6 +177,8 @@ class ActivatorTest:
         # Then
         offer = Offer.query.filter_by(activityUuid=offer_uuid).one()
         stock = Stock.query.filter_by(activityUuid=stock_uuid).one()
+        assert offer.activityUuid == offer_activity.uuid
+        assert stock.activityUuid == stock_activity.uuid
         assert stock.offerId == offer.id
 
     @with_delete
@@ -191,4 +213,19 @@ class ActivatorTest:
         # Then
         offer = Offer.query.filter_by(activityUuid=offer_uuid).one()
         stock = Stock.query.filter_by(activityUuid=stock_uuid).one()
+        assert offer.activityUuid == offer_activity2.uuid
+        assert stock.activityUuid == stock_activity.uuid
         assert stock.offerId == offer.id
+
+    @with_delete
+    def test_get_activity_uuid_of_a_relationship(self, app):
+        # Given
+        offer = Offer(name='bar', type='foo')
+        stock = Stock(price=3, offer=offer)
+        ApiHandler.save(offer, stock)
+
+        # When
+        offer_activity_uuid = stock.offerActivityUuid
+
+        # Then
+        assert offer_activity_uuid == offer.activityUuid
